import hashlib
import time
import uuid

import requests
from httpClient.SdkException import *
from httpClient.RequestMethod import *
from httpClient.BaseHttpRequest import *


class SaaSSdkClient:

    def __init__(self, serverUrl,agentAccessKey,agentAccessSecret,
                 accessKey=None,accessSecret=None):
        self.__serverUrl = serverUrl
        self.__agentAccessKey = agentAccessKey
        self.__agentAccessSecret = agentAccessSecret
        self.__accessKey = accessKey
        self.__accessSecret = accessSecret

    def set_accessKey(self, value):
        self.__accessKey = value

    def set_accessSecret(self, value):
        self.__accessSecret = value

    def request(self, request):

        if not isinstance(request, BaseHttpRequest):
            raise ValueError('request must implement baseHttpRequest!')

        params = request.get_params()
        method = params.get_method()
        url = self.__serverUrl + request.get_path()

        timestamp = str(int(time.time() * 1000))
        nonce = str(uuid.uuid1())

        headers = {'x-qys-open-agentsignature': to_md5(self.__agentAccessKey + self.__agentAccessSecret + timestamp + nonce),
                   'x-qys-open-agentaccesstoken': self.__agentAccessKey,
                   'x-qys-open-timestamp': timestamp,
                   'version': "PYTHON-3.2.4",
                   "x-qys-open-nonce": nonce}
        if self.__accessKey is not None:
            signature = to_md5(self.__accessKey + self.__accessSecret + timestamp + nonce)
            headers['x-qys-open-signature'] = signature
            headers['x-qys-open-accesstoken'] = self.__accessKey

        while True:
            if method == RequestMethod.POST:
                if params.is_json():
                    data = params.get_json()
                    headers['Content-Type'] = 'application/json'
                    response = requests.post(url, data=data, headers=headers)
                    print(str(response))
                else:
                    files = params.get_file()
                    data = params.get_params()
                    response = requests.post(url, data=data, headers=headers, files=files)
            elif method == RequestMethod.GET:
                data = params.get_params()
                response = requests.get(url, params=data, headers=headers)
            else:
                raise BaseSdkException(message="request method is invalid")

            if response.status_code == 200:
                return response.content
            else:
                raise HttpException(response.status_code, response.reason)


def to_md5(plain_text):
    md5 = hashlib.md5()
    md5.update(plain_text.encode('utf-8'))
    return md5.hexdigest()
